@@ -3,7 +3,7 @@
  *
  * This file is part of msmtp, an SMTP client.
  *
- * Copyright (C) 2000, 2003, 2004, 2005, 2006
+ * Copyright (C) 2000, 2003, 2004, 2005, 2006, 2007
  * Martin Lambers <marlam@marlam.de>
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -33,7 +33,6 @@
 #include <strings.h>
 #include <ctype.h>
 #include <errno.h>
-extern int errno;
 
 #ifdef HAVE_LIBGSASL
 # include <gsasl.h>
@@ -1028,11 +1027,6 @@ int smtp_auth(smtp_server_t *srv,
 	{
 	    auth_mech = "CRAM-MD5";
 	}
-	else if (gsasl_client_support_p(ctx, "NTLM") 
-		&& (srv->cap.flags & SMTP_CAP_AUTH_NTLM))
-	{
-	    auth_mech = "NTLM";
-	}
 #ifdef HAVE_TLS
 	else if (tls_is_active(&srv->tls))
 	{
@@ -1046,6 +1040,11 @@ int smtp_auth(smtp_server_t *srv,
 	    {
 		auth_mech = "LOGIN";
 	    }
+	    else if (gsasl_client_support_p(ctx, "NTLM") 
+	    	    && (srv->cap.flags & SMTP_CAP_AUTH_NTLM))
+	    {
+		auth_mech = "NTLM";
+	    }
 	}
 #endif /* HAVE_TLS */
     }
@@ -1424,12 +1423,14 @@ int smtp_send_envelope(smtp_server_t *srv,
 		if (dsn_return)
 		{
 		    e = smtp_send_cmd(srv, errstr, "MAIL FROM:<%s> RET=%s", 
-			    envelope_from, dsn_return);
+			    strcasecmp(envelope_from, "MAILER-DAEMON") == 0 
+			    ? "" : envelope_from, dsn_return);
 		}
 		else
 		{
 		    e = smtp_send_cmd(srv, errstr, "MAIL FROM:<%s>", 
-			    envelope_from);
+			    strcasecmp(envelope_from, "MAILER-DAEMON") == 0 
+			    ? "" : envelope_from);
 		}
 		if (e != SMTP_EOK)
 		{
